/******************************************************************************
* C Include: GetImage                                                         *
*                                                                             *
* File:                                                                       *
*    getimage.h                                                               *
*                                                                             *
* Description:                                                                *
*    Include file for the main GetImage function and it's associated          *
*    functions.                                                               *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/

#ifndef __GETIMAGE__
#define __GETIMAGE__

#include "astronmy.h"
#include "extrctn.h"
#include "extrtlst.h"
#include "extrtset.h"
#include "field.h"
#include "fieldlst.h"
#include "generic.h"
#include "hdrlist.h"
#include "header.h"
#include "image.h"
#include "imgsnlst.h"
#include "platelst.h"

/*********************
* GetImage constants *
*********************/

#define MOUNT_COMMAND           "DSS_MNT"
#define DISMOUNT_COMMAND        "DSS_DMNT"

#define SEARCH_RADIUS_DEGREES 5.0 /* Minium distance from a plate's center   *
                                   *    to a field for that plate to be      *
                                   *    considered a candidate plate for the *
                                   *    field                                */

#define CDROM_DEVICE            "DSS_DEV"
#define CDROM_MOUNT_POINT       "DSS_MTPT"
#define CDROM_LABEL_PREFIX      "DSS_LBPF"

/******************************
* GetImage derived data types *
******************************/
enum user_io { STDIN_FIELDS         ,  /* Input is fields, ie, RA and Dec   *
                                        *    read from stdin.               */
               STDIN_IMAGE_SECTIONS ,  /* Input is image sections, ie, X-Y  *
                                        *    read from stdin.               */


/*******************************
* GetImage include file page 2 *
*******************************/

               STDOUT_IMAGE_SECTIONS,  /* Output is image sections, ie, X-Y *
                                        *    written to stdout.             */
               FIELD_FILE           ,  /* Input is fields from a file.      */
               IMAGE_SECTION_FILE   }; /* Input/Output is image sections    *
                                        *   from/to a file.                 */
typedef enum user_io USER_IO;

/* Action to take when can't find image directory in batch mode */
enum error_action { SKIP, ABORT, QUERY, RETRY };
typedef enum error_action ERROR_ACTION;

/* User input is specifications for image data to extract, either fields
 * specified by right ascension and declination, or image section x-y positions
 * on a plate.
 *
 * User output will receive a description of every image section to be
 * extracted if the user input was fields.
 */

struct GetImage_Options /* Options obtained from the command line */
{
   STRING       Command_String       ; /* Command used to invoke getimage     */
   EQUINOX      Equinox              ; /* For input coordinates               */
   USER_IO      User_Input           ; /* How do we know what data to extract */
   STRING       User_Input_File_Name ; /* Source of extraction info.          */
   FILE        *User_Input_File      ; /* Pointer to the input file.          */
   USER_IO      User_Output          ; /* stdout out or a file.               */
   STRING       User_Output_File_Name; /* Where we put what extractions done  */
   FILE        *User_Output_File     ; /* Pointer to the output file.         */
   BOOL         Extract_All_Images   ; /* If FALSE, only get image from plate *
                                        *    closest to field.                */
   BOOL         Extract_Images       ; /* If FALSE, don't extract images,     *
                                        *    only extraction information      */
   IMAGE_FORMAT Format               ; /* FITS or HHD                         */
   BOOL         GEIS_Header_Only     ; /* Don't extract GEIS image            */
   ERROR_ACTION Batch_Error_Action   ; /* Specifies action to take if, when   *
                                        *    extracting images using fields   *
                                        *    read from a file, the image data *
                                        *    data directory cannot be found.  *
                                        *    The choices are to skip the      *
                                        *    current disc (AUTO_SKIP), abort  *
                                        *    GetImage (AUTO_ABORT) or query   *
                                        *    the user to insert the correct   *
                                        *    disc, skip the current disc or   *
                                        *    to abort GetImage (AUTO_QUERY).  */
  BOOL          dontUnmountOnExit    ; /* Don't attempt to dismount CD-ROM    *
                                        * when exiting GetImaeg               */
/*
 * These options are obtained from environment variables
 */
   STRING       Header_Directory   ; /* Directory where plate headers are */
   STRING       Header_Extension   ; /* Extension for header files        */
   STRING       Plate_List_File    ; /* Name of plate list file           */
   STRING       Image_Data_Root_Dir; /* Root directory for image data     */
   STRING       Mount_Command      ; /* Command to mount CD-ROM           */
   STRING       Dismount_Command   ; /* Command to dismount CD-ROM        */
   STRING       CDROM_Device       ; /* Device for CD-ROM drive           */
   STRING       CDROM_Mount_Point  ; /* Mount point for UNIX CD-ROMs      */
   STRING       CDROM_Label_Prefix ; /* Prefix for CD-ROM volume labels   */
};

/*******************************
* GetImage include file page 3 *
*******************************/

typedef struct GetImage_Options *OPTIONS;

/*********************************
* GetImage functions' prototypes *
*********************************/

void         Add_Field_to_Extraction_Set
                                      (/* FIELD         Field             ,
					  FIELD_LIST    Field_List        ,
					  PLATE_LIST    Master_Plate_List ,
					  IS_LIST       Image_Section_List,
					  HDR_LIST      Header_List       ,
					  X_SET         Extraction_Set    ,
					  OPTIONS       Options            */);

void         Close_User_Input         (/* OPTIONS       Options            */);

void         Close_User_Output        (/* OPTIONS       Options            */);

STRING       createDDS_CDROMlabel     (/* STRING        Label_Prefix      ,
					  int           Disc               */);

STRING       createVMS_CDROMdismountCommand
                                      (/* STRING        BaseCommand       ,
					  STRING        Device             */);

STRING       createVMS_CDROMmountCommand
                                      (/* STRING        BaseCommand       ,
					  STRING        Device             ,
					  STRING        volumeLabel        */);

STRING       createUNIX_CDROMdismountCommand
                                      (/* STRING        BaseCommand       ,
					  STRING        Device             */);

STRING       createUNIX_CDROMmountCommand
                                      (/* STRING        BaseCommand       ,
					  STRING        Device            ,
					  STRING        mountPoint         */);

void         Delete_Options           (/* OPTIONS       Options            */);

void         dismountCDROM            (/* STRING        BaseCommand       ,
					  STRING        Device             */);

void         Extract_Disc_List_HHH_Headers
                                      (/* X_LIST        Disc_List         ,
					  HDR_LIST      Header_List        */);

void         Extract_Disc_List_Images (/* X_LIST        Disc_List         ,
					  HDR_LIST      Header_List       ,
					  OPTIONS       Options            */);

void         Extract_Fields_from_File (/* PLATE_LIST    Plate_List        ,
					  OPTIONS       Options            */);
void         Extract_Fields_Interactively
                                      (/* PLATE_LIST    Plate_List        ,
					  OPTIONS       Options            */);



/*******************************
* GetImage include file page 4 *
*******************************/

void         Extract_Images_from_Set  (/* X_SET         Extraction_Set    ,
					  HDR_LIST      Header_List       ,
					  OPTIONS       Options            */);

BOOL         Extracting_Images        (/* OPTIONS       Options            */);

X_LIST       Get_Cand_Plates_Extraction_Info
                                      (/* FIELD         Field             ,
					  PLATE_LIST    Candidate_Plates  ,
					  OPTIONS       Options           ,
					  IS_LIST       Image_Section_List,
					  HDR_LIST      Header_List        */);

ERROR_ACTION Get_Disc_Error_Action    (/* int           Disc              ,
					  OPTIONS       Options            */);

void         Get_Extractions_from_File(/* PLATE_LIST    Plate_List        ,
					  OPTIONS       Options            */);

OPTIONS      Get_GetImage_Options     (/* int           argc              ,
					  char         *argv[]             */);

PLATE_LIST   Make_Candidate_Plate_List(/* PLATE_LIST    Plate_List        ,
					  FIELD         Field              */);

STRING       Make_GetImage_Image_File_Root
                                      (/* EXTRACTION    Extraction         */);

void         mountCDROM               (/* STRING        BaseCommand       ,
					  STRING        Device            ,
					  STRING        volumeLabel       ,
					  STRING        mountPoint         */);


void         Open_User_Input          (/* OPTIONS       Options            */);

void         Open_User_Output         (/* OPTIONS       Options            */);

void         Print_Help_and_Stop      (/* void                             */);

void         Print_Interactive_Instructions
                                      (/* OPTIONS       Options            */);

void         Print_Usage              (/* void                             */);

#endif
